package scanner

import (
	"git.akyoto.dev/cli/q/src/core"
	"git.akyoto.dev/cli/q/src/errors"
	"git.akyoto.dev/cli/q/src/fs"
	"git.akyoto.dev/cli/q/src/token"
)

// scanConst scans a block of constants.
func (s *Scanner) scanConst(file *fs.File, tokens token.List, i int) (int, error) {
	i++

	if tokens[i].Kind != token.Identifier {
		return i, errors.New(errors.ExpectedConstName, file, tokens[i].Position)
	}

	groupName := tokens[i].Text(file.Bytes)
	i++

	if tokens[i].Kind != token.BlockStart {
		return i, errors.New(errors.MissingBlockStart, file, tokens[i].Position)
	}

	i++

	for i < len(tokens) {
		switch tokens[i].Kind {
		case token.Identifier:
			name := tokens[i].Text(file.Bytes)
			i++

			s.constants <- &core.Constant{
				Name:  file.Package + "." + groupName + "." + name,
				Token: tokens[i],
				File:  file,
			}

		case token.BlockEnd:
			return i, nil
		}

		i++
	}

	return i, errors.New(errors.MissingBlockEnd, file, tokens[i].Position)
}
